==> musashi-core-1.0.4-pre6.tar.gz: musashi-core-1.0.4-pre6/cmd/xmltree.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : APIб                                                      */
/*============================================================================*/

#include <musashi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/parserInternals.h>
#include <libxml/encoding.h>
#include <iconv.h>
#include <errno.h>
#include <glob.h>
#include <stdarg.h>

#include <xmltreeHelp.h>
struct mssComHelp comHelp={
  "xmltree",      /* ޥ̾       */
  "1.0",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

#define UNDEF 0
#define MAX_NEST 32
#define LOCAL_BUF 256
#define EncMax MssFieldMaxLen  /*iconvǻȤʸĹ*/

/*----------------------------------------------------------------------------*/
/* ¤                                                                     */
/*----------------------------------------------------------------------------*/
struct Node {
  char  *element;               /*ʸ*/
  int    emptyTagFlg;
  char **atts;                  /*°ꥹ*/
  int    attCnt;
  struct Node  *parent;
  struct mssHash  *children;
};

typedef struct _XmlState {
  int level;
  int crFlg;
}XmlState;

/*----------------------------------------------------------------------------*/
/* Хѿ                                                             */
/*----------------------------------------------------------------------------*/
extern struct mssGlobalVariables mssGV;
extern xmlParserCtxtPtr ctxt;

static struct mssFPW    *fpw; /*ϥե빽¤*/
static iconv_t *icid;   /*iconv Ѵϥɥ*/
static char *inEnc=NULL; /*󥳡ǥ*/
static char *inVer=NULL; /*С*/
static struct Node *currentNode;
static struct Node *tree;

/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* ץ󥿥                                             */
    "o",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    OTFT,   /* ΥץΥȥ(Helpɽ)                         */
    OTFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
void *opt[]={&optINF,&optOTF,&optZIP,NULL};

/*----------------------------------------------------------------------------*/
/* ĥ꡼¤Ǥ°񤭽Ф                                           */
/*----------------------------------------------------------------------------*/
int Level=-1;
void printTree(struct Node *node){
  struct mssHashNode *hnode;
  struct mssXmlTag *xmlTag;
  int i;

  if(node->element !=NULL){
    mssWriteXmlIndent(Level,fpw);
    xmlTag=mssInitXmlTag(node->element,NULL);
    if(node->atts!=NULL){
      for(i=0;;i++){
        if(*(node->atts+2*i+0)==NULL) break;
        if(*(node->atts+2*i+1)==NULL) break;
        mssAddXmlTagAttributeStr(xmlTag,*(node->atts+2*i),
                                        *(node->atts+2*i+1),NULL);
      }
    } 

    if(node->emptyTagFlg){
      mssWriteXmlEmptyTag(xmlTag,icid,fpw);
      mssGV.outCnt++;
      mssWriteRet(fpw);
      return;
    }else{
      mssWriteXmlStartTag(xmlTag,icid,fpw);
      mssGV.outCnt++;
    }
    mssFreeXmlTag(xmlTag);

    if(node->children->cnt!=0){
      mssWriteRet(fpw);
    }
  }

  if(node->children==NULL) return;

  for(i=0; i<node->children->hashVal; i++){
    if( NULL != (hnode=*(node->children->node+i)) ){
      while(hnode!=NULL){
        Level++;
        printTree((struct Node *)(hnode->val.v.a));
        Level--;
        hnode=hnode->next;
      }
    }
  }

  if(node->element !=NULL){
    if(node->children->cnt!=0){
      mssWriteXmlIndent(Level,fpw);
    }

    xmlTag=mssInitXmlTag(node->element,NULL);
    mssWriteXmlEndTag(xmlTag,icid,fpw);
    mssFreeXmlTag(xmlTag);
    mssWriteRet(fpw);
  }
}

void freeTree(struct Node *node){
  struct mssHashNode *hnode;
  int i;

  if(node->children!=NULL){
    if(node->atts!=NULL){
      for(i=0; i<node->attCnt; i++){
        mssFree( *(node->atts+i*2+0) );
        mssFree( *(node->atts+i*2+1) );
      }
      mssFree(node->atts);
    }
    for(i=0; i<node->children->hashVal; i++){
      if( NULL != (hnode=*(node->children->node+i)) ){
        while(hnode!=NULL){
          freeTree((struct Node *)(hnode->val.v.a));
          hnode=hnode->next;
        }
      }
    }
    mssFreeHash(node->children);
  }

  if(node!=NULL) mssFree(node->element);
  mssFree(node);
}

/*----------------------------------------------------------------------------*/
/* SAX ϥɥ顼                                                             */
/*----------------------------------------------------------------------------*/
void start_doc(XmlState *state){

  inEnc=mssStrdup((char *)ctxt->input->encoding);
  inVer=mssStrdup((char *)ctxt->version);
  mssWriteXmlDeclaration( inVer, inEnc, fpw );
  if(inEnc==NULL) inEnc=mssStrdup("UTF-8");


  /*Ϥiconvץ*/
  icid=iconv_open(inEnc,"UTF-8");
  if((int)icid==-1){
      mssShowErrMsg("encoding type error in iconv_open");
      mssEnd(mssErrorNoDefault);
  }
}

void end_doc(XmlState *state){
  printTree(tree);
  freeTree(tree);

  if(icid!=NULL) iconv_close(icid);
}

/*ߤNodeλNode˻ꤵ줿elementСNode֤*/
/*ʤNULL֤                                           */
struct Node *getChildNode(struct Node *cn, char *element){
  struct mssHashNode *node;

  if( cn->element==NULL) return(NULL);
  if( cn->children==NULL) return(NULL);
  node=(struct mssHashNode *) mssHashMember(cn->children, element);

  if( NULL == node){
    return(NULL);
  }else{
    return((struct Node *)node->val.v.a);
  }
}


struct Node *addNode(struct Node *cn, char *element){
  MssValue node;

  /*Ρɤγ*/
  node.v.a=mssCalloc(sizeof(struct Node),"xmltree");

  /*̾,ΡɤΥɥ쥹childrenϿ*/
  mssHashInsert(cn->children, element, node);

  /*ΡɤλҥΡɤ*/
  ((struct Node *)(node.v.a))->children    = mssInitHash(11);
  ((struct Node *)(node.v.a))->element     = mssStrdup(element);
  ((struct Node *)(node.v.a))->emptyTagFlg = 0;
  ((struct Node *)(node.v.a))->parent      = cn;
  ((struct Node *)(node.v.a))->atts        = NULL;
  ((struct Node *)(node.v.a))->attCnt      = 0;

  return(node.v.a);
}

void addAtts(struct Node *node, char **atts){
  int i,j;
  int flg;
  
  if(atts!=NULL){
    i=0;
    while(*(atts+i)!=NULL){
    
      /*¸ߤƤcontinue*/
      flg=0;
      for(j=0; j<node->attCnt; j++){
        if( 0==strcmp(*(atts+i),*(node->atts+j*2)) ){
          flg=1;
          break;
        }
      }
      if(flg){
        i=i+2;
        continue;
      }

      /*ɲϿ*/
      node->atts = mssRealloc(node->atts,
        sizeof(char *)*((node->attCnt+2)*2),"xmltree");
      *(node->atts+node->attCnt*2+0)=mssStrdup(*(atts+i));
      *(node->atts+node->attCnt*2+1)=mssStrdup("");
      *(node->atts+node->attCnt*2+2)=NULL;
      *(node->atts+node->attCnt*2+3)=NULL;
      i=i+2;
      node->attCnt++;
    }
  }
}

/* start */
void start_element(XmlState *state, char *fullname, char **atts){
  struct Node *childNode;

  mssGV.inCnt++;
  if(state->level>MAX_NEST){
    mssShowErrMsg("nest level exceed (max=%d)" ,MAX_NEST);
    mssEnd(mssErrorNoDefault);
  }

  /*ȤλҥΡɤ*/
  childNode=getChildNode(currentNode,fullname);

  /*ȤλҥΡɤ̵ɲä*/
  if( childNode==NULL ){
    childNode=addNode(currentNode,fullname);
  }

  /*°ɲ*/
  addAtts(childNode,atts);

  /*νȤλҥΡɤ򥫥ȥΡɤȤ*/
  currentNode=childNode;

  state->level++;
}

/* end */
void end_element(XmlState *state, char *fullname, char **atts){
  state->level--;
  if(state->level<0){
    mssShowErrMsg("nest level reach to below 0");
    mssEnd(mssErrorNoDefault);
  }
  currentNode=currentNode->parent;
}

/*sax error handler*/
#include "saxerror.h"

static xmlSAXHandler SAXFunctions = {
    NULL, /* internalSubset */
    NULL, /* isStandalone */
    NULL, /* hasInternalSubset */
    NULL, /* hasExternalSubset */
    NULL, /* resolveEntity */
    NULL, /* getEntity */
    NULL, /* entityDecl */
    NULL, /* notationDecl */
    NULL, /* attributeDecl */
    NULL, /* elementDecl */
    NULL, /* unparsedEntityDecl */
    NULL, /* setDocumentLocator */
    (startDocumentSAXFunc)start_doc, /* startDocument */
    (endDocumentSAXFunc)end_doc, /* endDocument */
    (startElementSAXFunc)start_element, /* startElement */
    (endElementSAXFunc)end_element, /* endElement */
    NULL, /* reference */
    NULL, /* characters */
    NULL, /* ignorableWhitespace */
    NULL, /* processingInstruction */
    NULL, /* comment */
    (warningSAXFunc) xmlSaxErrEnd, /* xmlParserWarning */
    (errorSAXFunc) xmlSaxErrEnd, /* xmlParserError */
    (fatalErrorSAXFunc) xmlSaxErrEnd, /* xmlParserError */
    NULL, /* getParameterEntity */
};

int main(int argc, char *argv[]){

  XmlState         *state;

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* ʥʤɤν     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* إ                       */
  mssSetOption(opt,argc,argv);        /* ޥɥץ     */

  fpw=mssOpenFPW(optOTF.str,optZIP.set,0); /*ɸϥץ*/

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  tree=mssCalloc(sizeof(struct Node),"xmltree");
  tree->children = mssInitHash(11);
  currentNode=tree;

  state=mssCalloc(sizeof(XmlState),"xml2xt");

  if(optINF.set){
    ctxt=(xmlParserCtxtPtr)xmlCreateFileParserCtxt(optINF.str);
  }else{
    ctxt=(xmlParserCtxtPtr)xmlCreateFileParserCtxt("/dev/stdin");
  }
  if(!ctxt){
    mssShowErrMsg("not xml file\n");
    mssEnd(mssErrorNoDefault);
  }
  ctxt->sax=&SAXFunctions;

  ctxt->userData=state;
  xmlParseDocument(ctxt);
  ctxt->sax=NULL;
  xmlFreeParserCtxt(ctxt);

  mssFree(inEnc);
  mssFree(inVer);
  mssFree(state);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPW(fpw);       /*ϥեΥ   */
  mssFreeOption(opt);     /*ץΰ賫       */
  mssShowEndMsg();        /*λå           */
  mssEnd(mssExitSuccess); /*λ                     */
  return(0);              /* to avoid warning message*/
}

==> musashi-core-2.0.1.tar.gz: musashi-core-2.0.1/cmd/xmltree.c <==
/*============================================================================*/
/* 変更履歴                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : 新しいAPIに対応                                                      */
/*============================================================================*/

#include <musashi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/parserInternals.h>
#include <libxml/encoding.h>
#include <iconv.h>
#include <errno.h>
#include <glob.h>
#include <stdarg.h>

#include <xmltreeHelp.h>
struct mssComHelp comHelp={
  "xmltree",      /* コマンド名       */
  "1.0",          /* バージョン       */
  HELPT,          /* コマンドタイトル */
  HELPS,          /* 要約             */
  HELPE,          /* 利用例           */
  HELPR,          /* 参照コマンド     */
  HELPA,          /* 作者情報         */
  HELPB,          /* バグレポート情報 */
  HELPH           /* ホームページ     */
};

#define UNDEF 0
#define MAX_NEST 32
#define LOCAL_BUF 256
#define EncMax MssFieldMaxLen  /*iconvで使う出力用文字列長*/

/*----------------------------------------------------------------------------*/
/* 構造体                                                                     */
/*----------------------------------------------------------------------------*/
struct Node {
  char  *element;               /*要素文字列*/
  int    emptyTagFlg;
  char **atts;                  /*属性リスト*/
  int    attCnt;
  struct Node  *parent;
  struct mssHash  *children;
};

typedef struct _XmlState {
  int level;
  int crFlg;
}XmlState;

/*----------------------------------------------------------------------------*/
/* グローバル変数                                                             */
/*----------------------------------------------------------------------------*/
extern struct mssGlobalVariables mssGV;
extern xmlParserCtxtPtr ctxt;

static struct mssFPW    *fpw; /*出力ファイル構造体*/
static iconv_t *icid;   /*iconv用 変換ハンドラ*/
static char *inEnc=NULL; /*エンコーディング*/
static char *inVer=NULL; /*バージョン*/
static struct Node *currentNode;
static struct Node *tree;

/*============================================================================*/
/* オプション宣言＆定義                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* 入力ファイル                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* オプションタイプ                                             */
    "i",    /* キーワード(複数文字は不可)                                   */
    0,      /* 0:オプション, 1:必須                                         */
    1,      /* 指定可能の最大ファイル数                                     */
    0,      /*1:file not foundのエラーで終了しない 0:する                   */
    INFT,   /* このオプションのタイトル(Helpで表示)                         */
    INFC    /* このオプションのコメント(Helpで表示)                         */
  };

/*----------------------------------------------------------------------------*/
/* 出力ファイル                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* オプションタイプ                                             */
    "o",    /* キーワード(複数文字は不可)                                   */
    0,      /* 0:オプション, 1:必須                                         */
    OTFT,   /* このオプションのタイトル(Helpで表示)                         */
    OTFC    /* このオプションのコメント(Helpで表示)                         */
  };

/*----------------------------------------------------------------------------*/
/* 圧縮出力                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* オプションタイプ                                             */
    "z",    /* キーワード(複数文字は不可)                                   */
    0,      /* デフォルト(基本的には0) 常にonにしたいときは1にする          */
    ZIPT,   /* このオプションのタイトル(Helpで表示)                         */
    ZIPC    /* このオプションのコメント(Helpで表示)                         */
  };

/*----------------------------------------------------------------------------*/
/* オプションをまとめる                                                       */
/*----------------------------------------------------------------------------*/
void *opt[]={&optINF,&optOTF,&optZIP,NULL};

/*----------------------------------------------------------------------------*/
/* ツリー構造の要素と属性を書き出し                                           */
/*----------------------------------------------------------------------------*/
int Level=-1;
void printTree(struct Node *node){
  struct mssHashNode *hnode;
  struct mssXmlTag *xmlTag;
  int i;

  if(node->element !=NULL){
    mssWriteXmlIndent(Level,fpw);
    xmlTag=mssInitXmlTag(node->element,NULL);
    if(node->atts!=NULL){
      for(i=0;;i++){
        if(*(node->atts+2*i+0)==NULL) break;
        if(*(node->atts+2*i+1)==NULL) break;
        mssAddXmlTagAttributeStr(xmlTag,*(node->atts+2*i),
                                        *(node->atts+2*i+1),NULL);
      }
    } 

    if(node->emptyTagFlg){
      mssWriteXmlEmptyTag(xmlTag,icid,fpw);
      mssGV.outCnt++;
      mssWriteRet(fpw);
      return;
    }else{
      mssWriteXmlStartTag(xmlTag,icid,fpw);
      mssGV.outCnt++;
    }
    mssFreeXmlTag(xmlTag);

    if(node->children->cnt!=0){
      mssWriteRet(fpw);
    }
  }

  if(node->children==NULL) return;

  for(i=0; i<node->children->hashVal; i++){
    if( NULL != (hnode=*(node->children->node+i)) ){
      while(hnode!=NULL){
        Level++;
        printTree((struct Node *)(hnode->val.v.a));
        Level--;
        hnode=hnode->next;
      }
    }
  }

  if(node->element !=NULL){
    if(node->children->cnt!=0){
      mssWriteXmlIndent(Level,fpw);
    }

    xmlTag=mssInitXmlTag(node->element,NULL);
    mssWriteXmlEndTag(xmlTag,icid,fpw);
    mssFreeXmlTag(xmlTag);
    mssWriteRet(fpw);
  }
}

void freeTree(struct Node *node){
  struct mssHashNode *hnode;
  int i;

  if(node->children!=NULL){
    if(node->atts!=NULL){
      for(i=0; i<node->attCnt; i++){
        mssFree( *(node->atts+i*2+0) );
        mssFree( *(node->atts+i*2+1) );
      }
      mssFree(node->atts);
    }
    for(i=0; i<node->children->hashVal; i++){
      if( NULL != (hnode=*(node->children->node+i)) ){
        while(hnode!=NULL){
          freeTree((struct Node *)(hnode->val.v.a));
          hnode=hnode->next;
        }
      }
    }
    mssFreeHash(node->children);
  }

  if(node!=NULL) mssFree(node->element);
  mssFree(node);
}

/*----------------------------------------------------------------------------*/
/* SAX ハンドラー                                                             */
/*----------------------------------------------------------------------------*/
void start_doc(XmlState *state){

  inEnc=mssStrdup((char *)ctxt->input->encoding);
  inVer=mssStrdup((char *)ctxt->version);
  mssWriteXmlDeclaration( inVer, inEnc, fpw );
  if(inEnc==NULL) inEnc=mssStrdup("UTF-8");


  /*出力のiconvオープン*/
  icid=iconv_open(inEnc,"UTF-8");
  if((int)icid==-1){
      mssShowErrMsg("encoding type error in iconv_open");
      mssEnd(mssErrorNoDefault);
  }
}

void end_doc(XmlState *state){
  printTree(tree);
  freeTree(tree);

  if(icid!=NULL) iconv_close(icid);
}

/*現在のNodeの子Nodeに指定されたelementがあれば、そのNodeを返す*/
/*なければNULLを返す                                           */
struct Node *getChildNode(struct Node *cn, char *element){
  struct mssHashNode *node;

  if( cn->element==NULL) return(NULL);
  if( cn->children==NULL) return(NULL);
  node=(struct mssHashNode *) mssHashMember(cn->children, element);

  if( NULL == node){
    return(NULL);
  }else{
    return((struct Node *)node->val.v.a);
  }
}


struct Node *addNode(struct Node *cn, char *element){
  MssValue node;

  /*新しいノードの確保*/
  node.v.a=mssCalloc(sizeof(struct Node),"xmltree");

  /*エレメント名,新しいノードのアドレスをchildrenに登録*/
  mssHashInsert(cn->children, element, node);

  /*新しいノードの子ノードを初期化*/
  ((struct Node *)(node.v.a))->children    = mssInitHash(11);
  ((struct Node *)(node.v.a))->element     = mssStrdup(element);
  ((struct Node *)(node.v.a))->emptyTagFlg = 0;
  ((struct Node *)(node.v.a))->parent      = cn;
  ((struct Node *)(node.v.a))->atts        = NULL;
  ((struct Node *)(node.v.a))->attCnt      = 0;

  return(node.v.a);
}

void addAtts(struct Node *node, char **atts){
  int i,j;
  int flg;
  
  if(atts!=NULL){
    i=0;
    while(*(atts+i)!=NULL){
    
      /*既に存在していればcontinue*/
      flg=0;
      for(j=0; j<node->attCnt; j++){
        if( 0==strcmp(*(atts+i),*(node->atts+j*2)) ){
          flg=1;
          break;
        }
      }
      if(flg){
        i=i+2;
        continue;
      }

      /*追加登録*/
      node->atts = mssRealloc(node->atts,
        sizeof(char *)*((node->attCnt+2)*2),"xmltree");
      *(node->atts+node->attCnt*2+0)=mssStrdup(*(atts+i));
      *(node->atts+node->attCnt*2+1)=mssStrdup("");
      *(node->atts+node->attCnt*2+2)=NULL;
      *(node->atts+node->attCnt*2+3)=NULL;
      i=i+2;
      node->attCnt++;
    }
  }
}

/*エレメント start */
void start_element(XmlState *state, char *fullname, char **atts){
  struct Node *childNode;

  mssGV.inCnt++;
  if(state->level>MAX_NEST){
    mssShowErrMsg("nest level exceed (max=%d)" ,MAX_NEST);
    mssEnd(mssErrorNoDefault);
  }

  /*エレメントの子ノードを得る*/
  childNode=getChildNode(currentNode,fullname);

  /*エレメントの子ノードが無ければ追加する*/
  if( childNode==NULL ){
    childNode=addNode(currentNode,fullname);
  }

  /*属性を追加*/
  addAtts(childNode,atts);

  /*上の処理で得たエレメントの子ノードをカレントノードとする*/
  currentNode=childNode;

  state->level++;
}

/*エレメント end */
void end_element(XmlState *state, char *fullname, char **atts){
  state->level--;
  if(state->level<0){
    mssShowErrMsg("nest level reach to below 0");
    mssEnd(mssErrorNoDefault);
  }
  currentNode=currentNode->parent;
}

/*sax error handler*/
#include "saxerror.h"

static xmlSAXHandler SAXFunctions = {
    NULL, /* internalSubset */
    NULL, /* isStandalone */
    NULL, /* hasInternalSubset */
    NULL, /* hasExternalSubset */
    NULL, /* resolveEntity */
    NULL, /* getEntity */
    NULL, /* entityDecl */
    NULL, /* notationDecl */
    NULL, /* attributeDecl */
    NULL, /* elementDecl */
    NULL, /* unparsedEntityDecl */
    NULL, /* setDocumentLocator */
    (startDocumentSAXFunc)start_doc, /* startDocument */
    (endDocumentSAXFunc)end_doc, /* endDocument */
    (startElementSAXFunc)start_element, /* startElement */
    (endElementSAXFunc)end_element, /* endElement */
    NULL, /* reference */
    NULL, /* characters */
    NULL, /* ignorableWhitespace */
    NULL, /* processingInstruction */
    NULL, /* comment */
    (warningSAXFunc) xmlSaxErrEnd, /* xmlParserWarning */
    (errorSAXFunc) xmlSaxErrEnd, /* xmlParserError */
    (fatalErrorSAXFunc) xmlSaxErrEnd, /* xmlParserError */
    NULL, /* getParameterEntity */
};

int main(int argc, char *argv[]){

  XmlState         *state;

/*----------------------------------------------------------------------------*/
/* 前処理                                                                     */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* シグナル処理などの初期化     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* ヘルプ                       */
  mssSetOption(opt,argc,argv);        /* コマンドオプションの設定     */

  fpw=mssOpenFPW(optOTF.str,optZIP.set,0); /*標準出力オープン*/

/*----------------------------------------------------------------------------*/
/*メインルーチン                                                              */
/*----------------------------------------------------------------------------*/
  tree=mssCalloc(sizeof(struct Node),"xmltree");
  tree->children = mssInitHash(11);
  currentNode=tree;

  state=mssCalloc(sizeof(XmlState),"xml2xt");

  if(optINF.set){
    ctxt=(xmlParserCtxtPtr)xmlCreateFileParserCtxt(optINF.str);
  }else{
    ctxt=(xmlParserCtxtPtr)xmlCreateFileParserCtxt("/dev/stdin");
  }
  if(!ctxt){
    mssShowErrMsg("not xml file\n");
    mssEnd(mssErrorNoDefault);
  }
  ctxt->sax=&SAXFunctions;

  ctxt->userData=state;
  xmlParseDocument(ctxt);
  ctxt->sax=NULL;
  xmlFreeParserCtxt(ctxt);

  mssFree(inEnc);
  mssFree(inVer);
  mssFree(state);

/*----------------------------------------------------------------------------*/
/*フッター出力&終了処理                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPW(fpw);       /*出力ファイルのクローズ   */
  mssFreeOption(opt);     /*オプション領域開放       */
  mssShowEndMsg();        /*完了メッセージ           */
  mssEnd(mssExitSuccess); /*終了                     */
  return(0);              /* to avoid warning message*/
}
